@@ -26,6 +26,7 @@ CONFIG_NODE_MQTT = "mqtt"
 CONFIG_NODE_STATS = "stats"
 CONFIG_NODE_PORT = "port"
 
-DEFAULT_SERVER_HOST = "localhost"
+DEFAULT_SERVER_HOST = "127.0.0.1"
+DEFAULT_SERVER_PORT = 9654
 
 ENDPOINT_DEVICES = "devices"
