@@ -1,6 +1,6 @@
 import asyncio
 import logging
-from typing import List
+from typing import Dict
 
 import aiohttp
 from aiohttp import web
@@ -28,18 +28,18 @@ class Webserver:
         self.host = config.SERVER_HOST.value
         self.port = config.SERVER_PORT.value
 
-        self.clients = set()
+        self.clients = {}
 
         self.barcode_reader = barcode_reader
         self.barcode_reader.add_listener(self.on_barcode)
 
-        self.notifiers: List[BarcodeNotifier] = []
+        self.notifiers: Dict[str, BarcodeNotifier] = {}
         if config.HTTP_URL.value is not None:
             http_notifier = HttpNotifier(
                 config.HTTP_METHOD.value,
                 config.HTTP_URL.value,
                 config.HTTP_HEADERS.value)
-            self.notifiers.append(http_notifier)
+            self.notifiers["http"] = http_notifier
 
         if config.MQTT_HOST.value is not None:
             mqtt_notifier = MQTTNotifier(
@@ -52,19 +52,18 @@ class Webserver:
                 qos=config.MQTT_QOS.value,
                 retain=config.MQTT_RETAIN.value,
             )
-            self.notifiers.append(mqtt_notifier)
+            self.notifiers["mqtt"] = mqtt_notifier
 
     async def start(self):
         # start detecting and reading barcode scanners
         await self.barcode_reader.start()
         # start notifier queue processors
-        for notifier in self.notifiers:
+        for key, notifier in self.notifiers.items():
+            LOGGER.debug(f"Starting notifier: {key}")
             await notifier.start()
         LOGGER.info(f"Starting webserver on {self.config.SERVER_HOST.value}:{self.config.SERVER_PORT.value} ...")
 
-        app = web.Application(middlewares=[self.authentication_middleware])
-        app.add_routes(routes)
-
+        app = self.create_app()
         runner = aiohttp.web.AppRunner(app)
         await runner.setup()
         site = aiohttp.web.TCPSite(
@@ -77,13 +76,30 @@ class Webserver:
         # wait forever
         return await asyncio.Event().wait()
 
+    def create_app(self) -> web.Application:
+        app = web.Application(middlewares=[self.authentication_middleware])
+        app.add_routes(routes)
+        return app
+
     @middleware
     async def authentication_middleware(self, request, handler):
-        if X_Auth_Token not in request.headers.keys() \
-                or request.headers[X_Auth_Token] != self.config.SERVER_API_TOKEN.value:
+        if self.config.SERVER_API_TOKEN.value is not None and \
+                (X_Auth_Token not in request.headers.keys()
+                 or request.headers[X_Auth_Token] != self.config.SERVER_API_TOKEN.value):
             LOGGER.warning(f"Rejecting unauthorized connection: {request.host}")
             return web.HTTPUnauthorized()
 
+        if Client_Id not in request.headers.keys():
+            LOGGER.warning(f"Rejecting client without {Client_Id} header: {request.host}")
+            return web.HTTPBadRequest()
+
+        client_id = request.headers[Client_Id].lower().strip()
+
+        if self.clients.get(client_id, None) is not None:
+            LOGGER.warning(
+                f"Rejecting new connection of already connected client {request.headers[Client_Id]}: {request.host}")
+            return web.HTTPBadRequest()
+
         return await handler(self, request)
 
     @routes.get(f"/{ENDPOINT_DEVICES}")
@@ -96,18 +112,38 @@ class Webserver:
 
     @routes.get("/")
     async def websocket_handler(self, request):
+        client_id = request.headers[Client_Id].lower().strip()
+
         websocket = web.WebSocketResponse()
         await websocket.prepare(request)
 
-        self.clients.add(websocket)
-        client_count = len(self.clients)
-        WEBSOCKET_CLIENT_COUNT.set(client_count)
+        self.clients[client_id] = websocket
+        active_client_count = self.count_active_clients()
+        known_client_ids_count = len(self.clients.keys())
+        # TODO: report both the mount of currently connected clients, as well as known client ids
+        WEBSOCKET_CLIENT_COUNT.set(active_client_count)
 
-        notifier = WebsocketNotifier(websocket)
+        if client_id not in self.notifiers.keys():
+            LOGGER.debug(
+                f"New client connected: {client_id} (from {request.host})")
+
+            LOGGER.debug(f"Creating new notifier for client id: {client_id}")
+            notifier = WebsocketNotifier(websocket)
+            self.notifiers[client_id] = notifier
+        else:
+            LOGGER.debug(
+                f"Previously seen client reconnected: {client_id} (from {request.host})")
+
+        notifier = self.notifiers[client_id]
+        if isinstance(notifier, WebsocketNotifier):
+            notifier.websocket = websocket
+
+        if Drop_Event_Queue in request.headers.keys():
+            await notifier.drop_queue()
+
+        LOGGER.debug(f"Starting notifier: {client_id}")
         await notifier.start()
-        self.notifiers.append(notifier)
 
-        LOGGER.debug(f"New client connected: {request.host} Client count: {client_count}")
         try:
             async for msg in websocket:
                 if msg.type == aiohttp.WSMsgType.TEXT:
@@ -121,16 +157,24 @@ class Webserver:
         except Exception as e:
             LOGGER.exception(e)
         finally:
-            # TODO: this notifier should not be removed immediately, to provide queueing functionality
-            self.notifiers.remove(notifier)
+            # TODO: should we remove this notifier after some time?
+            LOGGER.debug(f"Stopping notifier: {client_id}")
             await notifier.stop()
 
-            self.clients.discard(websocket)
-            client_count = len(self.clients)
-            WEBSOCKET_CLIENT_COUNT.set(client_count)
-            LOGGER.debug(f"Client disconnected: {request.host}")
+            self.clients[client_id] = None
+            self.clients.pop(client_id)
+            active_client_count = self.count_active_clients()
+            WEBSOCKET_CLIENT_COUNT.set(active_client_count)
+            LOGGER.debug(f"Client disconnected: {client_id} (from {request.host})")
         return websocket
 
     async def on_barcode(self, event: BarcodeEvent):
-        for notifier in self.notifiers:
+        for key, notifier in self.notifiers.items():
             await notifier.add_event(event)
+
+    def count_active_clients(self):
+        """
+        Counts the number of clients with an active websocket connection
+        :return: number of active clients
+        """
+        return len(list(filter(lambda x: x[1] is not None, self.clients.items())))
