@@ -5,7 +5,7 @@ from tests import TestBase
 
 class BarcodeReaderTest(TestBase):
 
-    def test_initialization(self):
+    async def test_initialization(self):
         config = AppConfig()
         reader = BarcodeReader(config)
         self.assertIsNotNone(reader)
